@@ -3,7 +3,7 @@
  *
  * The query object - this file is part a of the C extension module.
  *
- * Copyright (c) 2020 by the PyGreSQL Development Team
+ * Copyright (c) 2022 by the PyGreSQL Development Team
  *
  * Please see the LICENSE.TXT file for specific restrictions.
  */
@@ -224,7 +224,7 @@ static PyObject* query_iter(queryObject *self)
 }
 
 /* __next__() method of the queryObject:
-   Returns the current current row as a tuple and moves to the next one. */
+  Returns the current row as a tuple and moves to the next one. */
 static PyObject *
 query_next(queryObject *self, PyObject *noargs)
 {
@@ -357,7 +357,7 @@ _query_build_field_info(PGresult *res, int col_num) {
 
 /* Get information on one or all fields of the query result. */
 static char query_fieldinfo__doc__[] =
-"fieldinfo() -- return info on field(s) in query";
+"fieldinfo([name]) -- return information about field(s) in query result";
 
 static PyObject *
 query_fieldinfo(queryObject *self, PyObject *args)
@@ -522,7 +522,7 @@ _query_row_as_dict(queryObject *self)
     return row_dict;
 }
 
-/* Return the current current row as a dict and move to the next one. */
+/* Return the current row as a dict and move to the next one. */
 static PyObject *
 query_next_dict(queryObject *self, PyObject *noargs)
 {
