@@ -11,23 +11,11 @@ from threading import Thread
 
 from pg import *
 
-# We need a database to test against.  If LOCAL_PyGreSQL.py exists we will
-# get our information from that.  Otherwise we use the defaults.
-dbname = 'unittest'
-dbhost = None
-dbport = 5432
-
-try:
-    from .LOCAL_PyGreSQL import *  # noqa: F401
-except (ImportError, ValueError):
-    try:
-        from LOCAL_PyGreSQL import *  # noqa: F401
-    except ImportError:
-        pass
+from .config import dbname, dbhost, dbport, dbuser, dbpasswd
 
 
 def open_db():
-    db = DB(dbname, dbhost, dbport)
+    db = DB(dbname, dbhost, dbport, user=dbuser, passwd=dbpasswd)
     db.query("SET DATESTYLE TO 'ISO'")
     db.query("SET TIME ZONE 'EST5EDT'")
     db.query("SET DEFAULT_WITH_OIDS=FALSE")
@@ -302,12 +290,12 @@ class UtilityTest(unittest.TestCase):
                 NotificationHandler, db)
             arg_dict = dict(event=None, called=False)
             self.notify_timeout = False
-            # Listen for 'event_1' with timeout of 10ms.
-            target = fut('event_1', self.notify_callback, arg_dict, 0.01)
+            # Listen for 'event_1' with timeout of 50ms.
+            target = fut('event_1', self.notify_callback, arg_dict, 0.05)
             thread = Thread(None, target)
             thread.start()
-            # Sleep 20ms, long enough to time out.
-            sleep(0.02)
+            # Sleep 250ms, long enough to time out.
+            sleep(0.25)
             # Verify that we've indeed timed out.
             self.assertFalse(arg_dict.get('called'))
             self.assertTrue(self.notify_timeout)
